// ignore_for_file: avoid_print

import 'dart:async';
import 'package:testing/src/models/model.dart';
import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../utils/const.dart';

// ignore: must_be_immutable
class PreViewScreen extends StatefulWidget {
  /// Baseurl use for apicall.
  String? baseUrl;

  ///This is description.
  String? description;

  ///This is callbackurl.
  String? callbackUrl;

  ///This is successurl.
  String? successUrl;

  ///This is notificationemail.
  String? notificationEmail;

  ///This is customeremail.
  String? customerEmail;

  ///This is satoshis.
  int? satoshis;

  ///This is reference.
  String? reference;

  ///This is publickey.
  String? publicKey;

  ///This is sucesscallback.
  Function(
    dynamic response,
  ) successCallback;

  ///This is failcallback.
  Function(
    dynamic response,
  ) failCallback;

  ///This is closecallback.
  Function(dynamic response) closeCallBack;

  PreViewScreen(
      {Key? key,
      required this.baseUrl,
      required this.failCallback,
      required this.successCallback,
      required this.closeCallBack,
      this.callbackUrl,
      this.customerEmail,
      this.description,
      this.notificationEmail,
      this.publicKey,
      this.reference,
      this.satoshis,
      this.successUrl})
      : super(key: key);

  @override
  State<PreViewScreen> createState() => _PreViewScreenState();
}

class _PreViewScreenState extends State<PreViewScreen> {
  ///This loading bool show circular indicator.
  bool loading = true;

  ///Here define modelclass object.
  ModelClass? modelClass;

  ///Create dio object for api call.
  Dio dio = Dio();

  ///This is timer for call api every x seconds.
  Timer timer = Timer(const Duration(seconds: 0), () {});

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance!.addPostFrameCallback((timeStamp) {
      _apicallForPayMent();
    });
  }

  @override
  void dispose() {
    super.dispose();
    if (timer.isActive) {
      ///Here cancle the timer.
      timer.cancel();
    }
  }

  ///This method check payment status at every 5 second.
  _apiCallForCheckPaymentStatus() {
    timer = Timer.periodic(const Duration(seconds: 5), (timer) async {
      try {
        Response response = await dio
            .get(widget.baseUrl! + checkout_status + modelClass!.data!.id!);

        print(widget.baseUrl! + checkout_status + modelClass!.data!.id!);

        if (response.data['data']['status'].toString().toLowerCase() ==
            "paid") {
          timer.cancel();
          widget.successCallback(response.data);
        }
      } on DioError catch (e) {
        timer.cancel();
        widget.failCallback(e.response!.data);
      }
    });
  }

  ///This method use for payment
  _apicallForPayMent() async {
    try {
      Response response = await dio.get(
        widget.baseUrl! + create_checkout,
        queryParameters: {
          "description": widget.description,
          "callbackUrl": widget.callbackUrl,
          "successUrl": widget.successUrl,
          "notificationEmail": widget.notificationEmail,
          "customerEmail": widget.customerEmail,
          "satoshis": widget.satoshis,
          "reference": widget.reference,
          "publicKey": widget.publicKey
        },
      );
      if (response.statusCode == 200 || response.statusCode == 201) {
        modelClass = ModelClass.fromJson(response.data);

        if (mounted) {
          setState(() {});
        }
        _apiCallForCheckPaymentStatus();
      } else {
        widget.failCallback(response.data);
        Navigator.pop(context);
      }
    } on DioError catch (e) {
      widget.failCallback(e.response!.data);
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    SystemChrome.setSystemUIOverlayStyle(const SystemUiOverlayStyle(
      statusBarColor: Colors.black,
      statusBarIconBrightness: Brightness.light, // For Android (dark icons)
      statusBarBrightness: Brightness.dark,
      // status bar color
    ));
    return Scaffold(
      body: SafeArea(
        child: Stack(
          alignment: Alignment.topRight,
          children: [
            if ((modelClass?.data?.previewLink ?? "").isNotEmpty)

              ///This webview use for show payment screen.
              WebView(
                initialUrl: modelClass?.data?.previewLink ?? "",
                onPageFinished: (val) {
                  loading = false;
                  setState(() {});
                },
                javascriptMode: JavascriptMode.unrestricted,
                zoomEnabled: false,
              ),
            if (loading)
              const Center(
                child: CircularProgressIndicator(),
              ),
            Padding(
              padding: const EdgeInsets.symmetric(horizontal: 10),
              child: IconButton(
                  onPressed: () {
                    Navigator.pop(context);
                    widget.closeCallBack("Close Call Back");
                  },
                  icon: const Icon(
                    Icons.close,
                    color: Colors.black,
                  )),
            ),
          ],
        ),
      ),
    );
  }
}
